import re
import string
import sys
import shutil
import StringIO

toolname = "update-licenses"
filename = "tmp.txt"
explode = True

nonstandard_licenses = {
    "xerox" : """

  Copyright (c) 1995, 1996 Xerox Corporation.
  All Rights Reserved.

  Use and copying of this software and preparation of derivative works
  based upon this software are permitted. Any copy of this software or
  of any derivative work must include the above copyright notice of   
  Xerox Corporation, this paragraph and the one after it.  Any
  distribution of this software or derivative works must comply with all
  applicable United States export control laws.

  This software is made available AS IS, and XEROX CORPORATION DISCLAIMS
  ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE  
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR    
  PURPOSE, AND NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY
  LIABILITY FOR DAMAGES RESULTING FROM THE SOFTWARE OR ITS USE IS
  EXPRESSLY DISCLAIMED, WHETHER ARISING IN CONTRACT, TORT (INCLUDING
  NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX CORPORATION IS ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGES.
""",

    "chiba-tokyo" : """
  Copyright (C) 1997-2001 Shigeru Chiba, Tokyo Institute of Technology.

  Permission to use, copy, distribute and modify this software and
  its documentation for any purpose is hereby granted without fee,
  provided that the above copyright notice appears in all copies and that
  both that copyright notice and this permission notice appear in
  supporting documentation.

  Shigeru Chiba makes no representations about the suitability of this
  software for any purpose.  It is provided \"as is\" without express or
  implied warranty.
""",
    "chiba-tsukuba" : """
  Copyright (C) 1997-2001 Shigeru Chiba, University of Tsukuba.

  Permission to use, copy, distribute and modify this software and
  its documentation for any purpose is hereby granted without fee,
  provided that the above copyright notice appears in all copies and that
  both that copyright notice and this permission notice appear in 
  supporting documentation.

  Shigeru Chiba makes no representations about the suitability of this
  software for any purpose.  It is provided \"as is\" without express or
  implied warranty.
"""
}

standard_license = """
  Permission to use, copy, distribute and modify this software and its  
  documentation for any purpose is hereby granted without fee, provided that
  the above copyright notice appears in all copies and that both that copyright
  notice and this permission notice appear in supporting documentation.
 
  %(owner)s make(s) no representations about the suitability of this
  software for any purpose. It is provided \"as is\" without express or implied
  warranty.
  
  Copyright (C) %(period)s %(owner)s
"""

def process(filename):
    f = open(filename)
    text = f.read()
    f.close()

    start = re.search("@beginlicenses@[ \t]*\n", text)
    if not start:
        print >>sys.stderr, filename + ": NO LICENSE: missing or invalid @beginliceses"
        return
    startpoint = start.end()
    
    end = re.search("@endlicenses@", text[startpoint:]);
    if not end:
        print >>sys.stderr, filename + ": NO LICENSE: missing or invalid @endlicenses"
        return
    
    endpoint = startpoint + end.start() 
    while endpoint > startpoint and text[endpoint-1] != "\n": 
        endpoint = endpoint - 1
    
    body = text[startpoint:endpoint]
    lines = string.split(body, "\n")
    licenses = []
    for line in lines:
        if re.search("@", line):
            decl = re.match("(.*)@license{(.*)}{(.*)}@", string.strip(line))
            if decl:
                licenses.append((decl.group(1), decl.group(2), decl.group(3)))
            else:
                print >>sys.stderr, filename + ": invalid @license...@"
                sys.exit(1)
    output = StringIO.StringIO()
    first = True
    for license in licenses:
        print >>output, "%s@license{%s}{%s}@" % license
    if explode:
        for license in licenses:
            (prefix,owner,period) = license
            if first: first = False
            else: print >>output, prefix + '  ' + 65 * '-'
            if owner in nonstandard_licenses:
                license_text = nonstandard_licenses[owner]
            else:
                if owner == "contributors":
                    owner = "Other Contributors (see file AUTHORS)"
                license_text = standard_license % {"owner":owner, "period":period}
            print >>output, string.join(
                map(lambda x: prefix + x, string.split(license_text, "\n")),
                "\n")
    result = text[0:startpoint] + output.getvalue() + text[endpoint:]
    if result != text:
        print >>sys.stderr, filename + ": modyfying"
        shutil.copy(filename, filename + "~")
        out = file(filename, "w")
        out.write(result)

for arg in sys.argv[1:]:
    if arg == "--help": 
        print """
    USAGE: %s [--explode|--implode] [FILE]...
    
    Finds @beginlicenses@ ... @endlicenses@ pair and processes
    text between them: looks for lines of the form
    
        <PREFIX>@license{<AUTHOR>}{<PERIOD>}@
        
    and expands each of them into the license text, prefixing
    each expanded line with PREFIX. If '--implode' is given,
    does not expand license text (at the same time removes any
    existing license text).
    
"""
        sys.exit(0)
    elif arg == "--implode": explode = False
    else:
        process(arg)

